Package: ssc
Type: Package
Title: Semi-Supervised Classification Methods
Version: 2.1-0
Authors@R: c(person("Mabel", "González", role = "aut",
                    email = "mabelc@correo.ugr.es",
                    comment = c(ORCID = "0000-0003-0152-444X")),
             person("Osmani", "Rosado-Falcón", role = "aut",
                    email = "osmanir@uclv.cu",
                    comment = c(ORCID = "0000-0002-2639-3354")),
             person("José Daniel", "Rodríguez", role = "aut",
                    email = "josedaniel@uclv.cu",
                    comment = c(ORCID = "0000-0002-8489-4106")),
             person("Christoph", "Bergmeir", role = c("ths", "cre"),
                    email = "c.bergmeir@decsai.ugr.es",
                    comment = c(ORCID = "0000-0002-3665-9021")),
             person("Isaac", "Triguero", role = "ctb",
                    email = "isaac.triguero@nottingham.ac.uk",
                    comment = c(ORCID = "0000-0002-0150-0651")),
             person("José Manuel", "Benítez", role = "ths",
                    email = "j.m.benitez@decsai.ugr.es",
                    comment = c(ORCID = "0000-0002-2346-0793"))
             )
Description: Provides a collection of self-labeled techniques for semi-supervised 
    classification. In semi-supervised classification, both labeled and unlabeled
    data are used to train a classifier. This learning paradigm has obtained promising
    results, specifically in the presence of a reduced set of labeled examples. 
    This package implements a collection of self-labeled techniques to construct a
    classification model. This family of techniques enlarges the original labeled set 
	using the most confident predictions to classify unlabeled data. The techniques 
	implemented can be applied to classification problems in several domains by the 
	specification of a supervised base classifier. At low ratios of labeled data, it 
	can be shown to perform better than classical supervised classifiers.
Depends: R (>= 3.2.3)
Imports: stats, proxy
Suggests: caret, e1071, C50, kernlab, testthat, timeDate, stringi,
        R.rsp
VignetteBuilder: R.rsp
License: GPL (>= 3)
URL: https://github.com/mabelc/SSC
BugReports: https://github.com/mabelc/SSC/issues
Encoding: UTF-8
RoxygenNote: 6.1.0
NeedsCompilation: no
Packaged: 2019-12-15 20:56:23 UTC; bergmeir
Author: Mabel González [aut] (<https://orcid.org/0000-0003-0152-444X>),
  Osmani Rosado-Falcón [aut] (<https://orcid.org/0000-0002-2639-3354>),
  José Daniel Rodríguez [aut] (<https://orcid.org/0000-0002-8489-4106>),
  Christoph Bergmeir [ths, cre] (<https://orcid.org/0000-0002-3665-9021>),
  Isaac Triguero [ctb] (<https://orcid.org/0000-0002-0150-0651>),
  José Manuel Benítez [ths] (<https://orcid.org/0000-0002-2346-0793>)
Maintainer: Christoph Bergmeir <c.bergmeir@decsai.ugr.es>
Repository: CRAN
Date/Publication: 2019-12-15 21:30:02 UTC
Built: R 4.6.0; ; 2026-04-25 22:34:07 UTC; unix
