# Generated by roxygen2: do not edit by hand

S3method(predict,OneNN)
S3method(predict,coBC)
S3method(predict,democratic)
S3method(predict,selfTraining)
S3method(predict,setred)
S3method(predict,snnrce)
S3method(predict,snnrceG)
S3method(predict,triTraining)
export(coBC)
export(coBCCombine)
export(coBCG)
export(democratic)
export(democraticCombine)
export(democraticG)
export(oneNN)
export(selfTraining)
export(selfTrainingG)
export(setred)
export(setredG)
export(snnrce)
export(triTraining)
export(triTrainingCombine)
export(triTrainingG)
importFrom(stats,predict)
